@@ -5,6 +5,7 @@ The main interface to work with YaraRule objects
 import logging
 import collections
 
+
 from yarabuilder.yararule import YaraRule
 
 
@@ -43,6 +44,16 @@ class YaraBuilder:
         if rule_name not in self.yara_rules:
             raise KeyError('Rule "{0}" doesn\'t exist'.format(rule_name))
 
+    def get_yara_rule_names(self):
+        """
+        Method to return all Yara rule names in a YaraBuilder object
+
+        Returns:
+            list: list of strings of the Yara rule names
+        """
+
+        return list(self.yara_rules.keys())
+
     def create_rule(self, rule_name):
         """
         Create a new YaraRule object in the YaraBuilder
@@ -101,10 +112,12 @@ class YaraBuilder:
             else:
                 meta_type = "text"
 
-        self.logger.debug("Using meta_type %s", meta_type)
+        self.logger.debug("Using meta_type %s for %s", meta_type, str(value))
         self.yara_rules[rule_name].meta.add_meta(name, value, meta_type=meta_type)
 
-    def add_text_string(self, rule_name, value, name=None, modifiers=None):
+    def add_text_string(
+        self, rule_name, value, name=None, modifiers=None, newline_after=False
+    ):
         """
         Wrapper method to add a text string (e.g. $ = "test") to the specified rule_name
 
@@ -114,10 +127,14 @@ class YaraBuilder:
             name (str, optional): the optional name of the string
                 (if not provided will add as anonymous string)
             modifiers (:obj:`list` of :obj:`str`, optional): any modifiers to add to the string
+            newline_after (bool, optional): bool to determine if there should be
+                an extra newline after the string
         """
         self._add_string(rule_name, value, "text", name=name, modifiers=modifiers)
 
-    def add_hex_string(self, rule_name, value, name=None, modifiers=None):
+    def add_hex_string(
+        self, rule_name, value, name=None, modifiers=None, newline_after=False
+    ):
         """
         Wrapper method to add a hex string (e.g. $ = {DE AD BE EF}) to the specified rule_name
 
@@ -127,6 +144,8 @@ class YaraBuilder:
             name (str, optional): the name of the string
                 (if not provided will add as anonymous string)
             modifiers (:obj:`list` of :obj:`str`, optional): any modifiers to add to the string
+            newline_after (bool, optional): bool to determine if there should be
+                an extra newline after the string
         """
 
         if value.startswith("{") and value.endswith("}"):
@@ -134,7 +153,9 @@ class YaraBuilder:
 
         self._add_string(rule_name, value, "hex", name=name, modifiers=modifiers)
 
-    def add_regex_string(self, rule_name, value, name=None, modifiers=None):
+    def add_regex_string(
+        self, rule_name, value, name=None, modifiers=None, newline_after=False
+    ):
         """
         Wrapper method to add a regex string (e.g. $ = /test[0-9]{2}/) to the specified rule_name
 
@@ -144,14 +165,46 @@ class YaraBuilder:
             name (str, optional): the name of the string
                 (if not provided will add as anonymous string)
             modifiers (:obj:`list` of :obj:`str`, optional): any modifiers to add to the string
+            newline_after (bool, optional): bool to determine if there should be
+                an extra newline after the string
         """
 
+        regex_flags = None
+        valid_regex_flags = "iu"
+
         if value.startswith("/") and value.endswith("/"):
             value = value[1:-1]
 
-        self._add_string(rule_name, value, "regex", name=name, modifiers=modifiers)
+        elif value.startswith("/") and not value.endswith("/"):
+            potential_flags = value.split("/")[-1]
 
-    def _add_string(self, rule_name, value, str_type, name=None, modifiers=None):
+            for potential_flag in potential_flags:
+                if potential_flag not in valid_regex_flags:
+                    raise ValueError('Invalid regex flag: "{0}"'.format(potential_flag))
+
+            regex_flags = potential_flags
+
+            value = "/".join(value.split("/")[:-1])[1:]
+
+        self._add_string(
+            rule_name,
+            value,
+            "regex",
+            name=name,
+            modifiers=modifiers,
+            regex_flags=regex_flags,
+        )
+
+    def _add_string(
+        self,
+        rule_name,
+        value,
+        str_type,
+        name=None,
+        modifiers=None,
+        regex_flags=None,
+        newline_after=False,
+    ):
         """
         Generic method to add a string based on the wrapper method call
 
@@ -162,6 +215,8 @@ class YaraBuilder:
             name (str, optional): the name of the string
                 (if not provided will add as anonymous string):
             modifiers (:obj:`list` of :obj:`str`, optional): any modifiers to add to the string
+            regex_flags (str, optional): any regex flags to be applied to a regex string
+            newline_after (bool, optional): bool to determine if there should be an extra newline after the string
         """
 
         if modifiers is None:
@@ -170,12 +225,19 @@ class YaraBuilder:
 
         if name:
             self.yara_rules[rule_name].strings.add_string(
-                name, value, str_type=str_type
+                name,
+                value,
+                str_type=str_type,
+                regex_flags=regex_flags,
+                newline_after=newline_after,
             )
 
         else:
             name = self.yara_rules[rule_name].strings.add_anonymous_string(
-                value, str_type=str_type
+                value,
+                str_type=str_type,
+                regex_flags=regex_flags,
+                newline_after=newline_after,
             )
 
         self._modifier_handler(rule_name, name, modifiers)
@@ -272,10 +334,10 @@ class YaraBuilder:
             first_rulename = list(self.yara_rules)[0]
 
             for rule_name in list(self.yara_rules)[1:]:
-                for import_str in self.yara_rules[rule_name].imports.imports:
+                for import_str in self.yara_rules[rule_name].imports.get_yara_imports():
                     self.add_import(first_rulename, import_str)
 
-                self.yara_rules[rule_name].imports.imports = []
+                self.yara_rules[rule_name].imports.set_yara_imports([])
 
         for rule in self.yara_rules.values():
             self.logger.debug("Building %s...", rule.rule_name)
@@ -318,7 +380,17 @@ def main():  # pragma: no cover
     Method to test if running the module from the command line
     """
 
-    yara_builder = YaraBuilder()
+    logger = logging.getLogger("yarabuilder_logger")
+    ch = logging.StreamHandler()
+    ch.setLevel(logging.DEBUG)
+
+    logger.setLevel(logging.DEBUG)
+
+    formatter = logging.Formatter("%(levelname)s:%(name)s:%(message)s")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
+    yara_builder = YaraBuilder(logger=logger)
 
     yara_builder.create_rule("test_rule1")
     yara_builder.add_meta("test_rule1", "test_name", "test_value")
