@@ -48,24 +48,14 @@ class YaraCommentEnabledClass:
         """
         if self.yara_comment.above:
             for above_comment in reversed(self.yara_comment.above):
-                raw = "// %s\n%s%s%s" % (
-                    above_comment,
-                    whitespace,
-                    whitespace,
-                    raw,
-                )
+                raw = "// %s\n%s%s%s" % (above_comment, whitespace, whitespace, raw,)
 
         if self.yara_comment.inline:
             raw = "%s // %s" % (raw, self.yara_comment.inline)
 
         if self.yara_comment.below:
             for below_comment in self.yara_comment.below:
-                raw = "%s\n%s%s// %s" % (
-                    raw,
-                    whitespace,
-                    whitespace,
-                    below_comment,
-                )
+                raw = "%s\n%s%s// %s" % (raw, whitespace, whitespace, below_comment,)
 
         return raw
 
@@ -345,7 +335,15 @@ class YaraString(YaraCommentEnabledClass):
         yara_comment (YaraComment): the comment associated with this entry
     """
 
-    def __init__(self, name, value, str_type="text", is_anonymous=False):
+    def __init__(
+        self,
+        name,
+        value,
+        str_type="text",
+        is_anonymous=False,
+        regex_flags=None,
+        newline_after=False,
+    ):
         """
         Constructor for YaraString
 
@@ -353,13 +351,18 @@ class YaraString(YaraCommentEnabledClass):
             name (str): the name of the string
             value (str): the value of the string
             str_type (str, optional): the type of the string ("text", "hex", or "regex")
+            regex_flags(str, optional): any regex flags to be applied to a regex string
             is_anonymous (bool, optional): bool set to False by default
+            newline_after (bool, optional): bool to determine if there should
+                be an extra newline after the string
         """
         self.name = name
         self.value = value
         self.str_type = str_type
         self.modifiers = []
         self.is_anonymous = is_anonymous
+        self.regex_flags = regex_flags
+        self.newline_after = newline_after
         self.raw_string = None
         self.yara_comment = YaraComment()
 
@@ -384,7 +387,10 @@ class YaraString(YaraCommentEnabledClass):
             self.raw_string += "{%s}" % self.value
 
         elif self.str_type == "regex":
-            self.raw_string += "/%s/" % self.value
+            if self.regex_flags:
+                self.raw_string += "/%s/%s" % (self.value, self.regex_flags)
+            else:
+                self.raw_string += "/%s/" % self.value
 
         if self.modifiers:
             for modifier in self.modifiers:
@@ -392,6 +398,9 @@ class YaraString(YaraCommentEnabledClass):
 
         self.raw_string = self.build_comments(self.raw_string, whitespace=whitespace)
 
+        if self.newline_after:
+            self.raw_string += "\n"
+
     def get_yara_string(self):
         """
         Method to return a structured POD version of YaraString
@@ -410,6 +419,9 @@ class YaraString(YaraCommentEnabledClass):
         if self.modifiers:
             yara_string["modifiers"] = self.modifiers
 
+        if self.regex_flags:
+            yara_string["regex_flags"] = self.regex_flags
+
         if (
             self.yara_comment.above
             or self.yara_comment.inline
@@ -443,6 +455,9 @@ class YaraString(YaraCommentEnabledClass):
         if "comment" in yara_string:
             self.yara_comment.set_yara_comment(yara_string["comment"])
 
+        if "regex_flags" in yara_string:
+            self.regex_flags = yara_string["regex_flags"]
+
 
 class YaraStrings:
     """
@@ -485,7 +500,9 @@ class YaraStrings:
 
         return str_type
 
-    def add_string(self, name, value, str_type="text"):
+    def add_string(
+        self, name, value, str_type="text", regex_flags=None, newline_after=False
+    ):
         """
         Add a named string to the YaraStrings object
 
@@ -493,22 +510,36 @@ class YaraStrings:
             name (str): name of the string
             value (str): the string
             str_type (str, optional): the type of the string ("text", "hex", "regex")
+            regex_flags (str, optional): any regex flags to be applied to a regex string
+            newline_after (bool, optional): bool to determine if there should
+                be an extra newline after the string
         """
         if name in self.strings:
             raise ValueError('String with name "{0}" already exists'.format(name))
 
         str_type = self._invalid_str_type_handler(str_type)
 
-        self.strings[name] = YaraString(name, value, str_type=str_type)
+        self.strings[name] = YaraString(
+            name,
+            value,
+            str_type=str_type,
+            regex_flags=regex_flags,
+            newline_after=newline_after,
+        )
         self.number_of_strings += 1
 
-    def add_anonymous_string(self, value, str_type="text"):
+    def add_anonymous_string(
+        self, value, str_type="text", regex_flags=None, newline_after=False
+    ):
         """
         Add an anonymous string to the YaraStrings object
 
         Args:
             value (str): the string
             str_type (str, optional): the type of the string ("text", "hex", "regex")
+            regex_flags (str, optional): any regex flags to be applied to a regex string
+            newline_after (bool, optional): bool to determine if there should
+                be an extra newline after the string
 
         Returns:
             str: the generated name of the string for later handling
@@ -517,7 +548,12 @@ class YaraStrings:
 
         name = "@anon%d" % self.number_of_anonymous_strings
         self.strings[name] = YaraString(
-            name, value, str_type=str_type, is_anonymous=True
+            name,
+            value,
+            str_type=str_type,
+            is_anonymous=True,
+            regex_flags=regex_flags,
+            newline_after=newline_after,
         )
         self.number_of_strings += 1
         self.number_of_anonymous_strings += 1
@@ -549,6 +585,8 @@ class YaraStrings:
         """
         Build each string object
         """
+        self.raw_strings = []
+
         for yara_string in self.strings.values():
             yara_string.build_string()
             self.raw_strings.append(yara_string.raw_string)
@@ -668,6 +706,8 @@ class YaraImports:
         """
         Build the imports section into one string
         """
+        self.raw_imports = ""
+
         for import_str in self.imports:
             self.raw_imports += 'import "%s"\n' % import_str
 
@@ -888,6 +928,8 @@ class YaraRule:
                 '"{0}" has no raw_condition, cannot build rule'.format(self.rule_name)
             )
 
+        self.raw_rule = ""
+
         self.logger.debug("Building rule header for %s...", self.rule_name)
         self.raw_rule = self.build_rule_header(self.raw_rule)
 
@@ -964,7 +1006,18 @@ def main():  # pragma: no cover
     """
     Method to test if running the module from the command line
     """
-    rule = YaraRule("command_line_rule")
+
+    logger = logging.getLogger("yararule_logger")
+    ch = logging.StreamHandler()
+    ch.setLevel(logging.DEBUG)
+
+    logger.setLevel(logging.DEBUG)
+
+    formatter = logging.Formatter("%(levelname)s:%(name)s:%(message)s")
+    ch.setFormatter(formatter)
+    logger.addHandler(ch)
+
+    rule = YaraRule("command_line_rule", logger=logger)
     rule.condition.add_raw_condition("filesize > 0")
     rule.tags.add_tag("test1")
     rule.tags.add_tag("test2")
@@ -974,7 +1027,7 @@ def main():  # pragma: no cover
     rule.meta.add_meta("test_meta", "test2")
     rule.meta.meta["test_meta"][0].add_comment("hi", position="above")
     rule.meta.meta["test_meta"][1].add_comment("there", position="above")
-    rule.strings.add_string("test_string_text", "string_text_val")
+    rule.strings.add_string("test_string_text", "string_text_val", newline_after=True)
     rule.strings.add_modifier("test_string_text", "ascii")
     rule.strings.strings["test_string_text"].add_comment("above comment")
     rule.strings.strings["test_string_text"].add_comment(
